@@ -818,7 +818,7 @@ account_chooser_update_iter (EmpathyAccountChooser *chooser,
 
 	data = g_slice_new0 (FilterResultCallbackData);
 	data->chooser = chooser;
-	data->account = account;
+	data->account = g_object_ref (account);
 	data->iter = g_memdup (iter, sizeof (GtkTreeIter));
 
 	if (priv->filter)
@@ -826,6 +826,8 @@ account_chooser_update_iter (EmpathyAccountChooser *chooser,
 			      (gpointer) data, priv->filter_data);
 	else
 		account_chooser_filter_ready_cb (TRUE, (gpointer) data);
+
+	g_object_unref (account);
 }
 
 static void
