@@ -65,6 +65,7 @@ typedef struct {
   GtkWidget *radiobutton_reuse;
 
   gboolean simple;
+  gboolean enabled;
 
   gboolean contains_pending_changes;
 
@@ -88,6 +89,12 @@ typedef struct {
   GtkWidget *param_account_widget;
   GtkWidget *param_password_widget;
 
+  gboolean automatic_change;
+  GtkWidget *remember_password_widget;
+
+  /* Used only for IRC accounts */
+  EmpathyIrcNetworkChooser *irc_network_chooser;
+
   gboolean dispose_run;
 } EmpathyAccountWidgetPriv;
 
@@ -97,6 +104,7 @@ enum {
   PROP_SIMPLE,
   PROP_CREATING_ACCOUNT,
   PROP_OTHER_ACCOUNTS_EXIST,
+  PROP_ENABLED,
 };
 
 enum {
@@ -106,6 +114,8 @@ enum {
   LAST_SIGNAL
 };
 
+static void account_widget_apply_and_log_in (EmpathyAccountWidget *);
+
 static guint signals[LAST_SIGNAL] = { 0 };
 
 #define GET_PRIV(obj) EMPATHY_GET_PRIV (obj, EmpathyAccountWidget)
@@ -181,6 +191,11 @@ static void
 account_widget_entry_changed_cb (GtkEditable *entry,
     EmpathyAccountWidget *self)
 {
+  EmpathyAccountWidgetPriv *priv = GET_PRIV (self);
+
+  if (priv->automatic_change)
+    return;
+
   account_widget_entry_changed_common (self, GTK_ENTRY (entry), FALSE);
   empathy_account_widget_changed (self);
 }
@@ -354,6 +369,16 @@ password_entry_changed_cb (GtkEditable *entry,
       GTK_ENTRY_ICON_SECONDARY, !EMP_STR_EMPTY (str));
 }
 
+static void
+password_entry_activated_cb (GtkEntry *entry,
+    EmpathyAccountWidget *self)
+{
+    EmpathyAccountWidgetPriv *priv = GET_PRIV (self);
+
+    if (gtk_widget_get_sensitive (priv->apply_button))
+        account_widget_apply_and_log_in (self);
+}
+
 void
 empathy_account_widget_setup_widget (EmpathyAccountWidget *self,
     GtkWidget *widget,
@@ -430,6 +455,8 @@ empathy_account_widget_setup_widget (EmpathyAccountWidget *self,
               G_CALLBACK (clear_icon_released_cb), self);
           g_signal_connect (widget, "changed",
               G_CALLBACK (password_entry_changed_cb), self);
+          g_signal_connect (widget, "activate",
+              G_CALLBACK (password_entry_activated_cb), self);
         }
 
       g_signal_connect (widget, "changed",
@@ -488,11 +515,34 @@ empathy_account_widget_setup_widget (EmpathyAccountWidget *self,
     }
 }
 
+static GHashTable *
+build_translated_params (void)
+{
+  GHashTable *hash;
+
+  hash = g_hash_table_new (g_str_hash, g_str_equal);
+  g_hash_table_insert (hash, "account", _("Account"));
+  g_hash_table_insert (hash, "password", _("Password"));
+  g_hash_table_insert (hash, "server", _("Server"));
+  g_hash_table_insert (hash, "port", _("Port"));
+
+  return hash;
+}
+
 static gchar *
 account_widget_generic_format_param_name (const gchar *param_name)
 {
   gchar *str;
   gchar *p;
+  static GHashTable *translated_params = NULL;
+
+  if (G_UNLIKELY (translated_params == NULL))
+    translated_params = build_translated_params ();
+
+  /* Translate most common parameters */
+  str = g_hash_table_lookup (translated_params, param_name);
+  if (str != NULL)
+    return g_strdup (str);
 
   str = g_strdup (param_name);
 
@@ -708,18 +758,23 @@ account_widget_account_enabled_cb (GObject *source_object,
   g_object_unref (widget);
 }
 
+static void account_widget_applied_continue (EmpathyAccountWidget *widget,
+    gboolean reconnect_required);
+static void account_widget_applied_additional_cb (GObject *widget,
+    GAsyncResult *result, gpointer user_data);
+
 static void
 account_widget_applied_cb (GObject *source_object,
     GAsyncResult *res,
     gpointer user_data)
 {
   GError *error = NULL;
-  TpAccount *account;
   EmpathyAccountSettings *settings = EMPATHY_ACCOUNT_SETTINGS (source_object);
   EmpathyAccountWidget *widget = EMPATHY_ACCOUNT_WIDGET (user_data);
-  EmpathyAccountWidgetPriv *priv = GET_PRIV (widget);
+  gboolean reconnect_required;
 
-  empathy_account_settings_apply_finish (settings, res, &error);
+  empathy_account_settings_apply_finish (settings, res, &reconnect_required,
+      &error);
 
   if (error != NULL)
     {
@@ -728,6 +783,44 @@ account_widget_applied_cb (GObject *source_object,
       return;
     }
 
+  DEBUG ("First stage apply complete: reconnect_required = %s",
+      reconnect_required ? "yes" : "no");
+
+  /* if there is additional work for this widget, do that now */
+  if (widget->ui_details->additional_apply_async != NULL)
+    widget->ui_details->additional_apply_async (widget,
+        account_widget_applied_additional_cb,
+        GUINT_TO_POINTER (reconnect_required));
+  else
+    account_widget_applied_continue (widget, reconnect_required);
+}
+
+static void
+account_widget_applied_additional_cb (GObject *obj,
+    GAsyncResult *result,
+    gpointer user_data)
+{
+  EmpathyAccountWidget *widget = EMPATHY_ACCOUNT_WIDGET (obj);
+  GSimpleAsyncResult *simple = G_SIMPLE_ASYNC_RESULT (result);
+  gboolean reconnect_required = GPOINTER_TO_UINT (user_data);
+
+  /* we don't care about the error status, the async method can report it,
+   * just get the result */
+  reconnect_required |= g_simple_async_result_get_op_res_gboolean (simple);
+
+  DEBUG ("Second stage apply complete: reconnect_required = %s",
+      reconnect_required ? "yes" : "no");
+
+  account_widget_applied_continue (widget, reconnect_required);
+}
+
+static void
+account_widget_applied_continue (EmpathyAccountWidget *widget,
+    gboolean reconnect_required)
+{
+  EmpathyAccountWidgetPriv *priv = GET_PRIV (widget);
+  TpAccount *account;
+
   account = empathy_account_settings_get_account (priv->settings);
 
   if (account != NULL)
@@ -743,23 +836,21 @@ account_widget_applied_cb (GObject *source_object,
               account_widget_account_enabled_cb, widget);
           g_signal_emit (widget, signals[ACCOUNT_CREATED], 0, account);
         }
-      else if (priv->enabled_checkbox != NULL)
+      else
         {
-          gboolean enabled_checked;
+          /* If the account was offline, we always want to try reconnecting,
+           * to give it a chance to connect if the previous params were wrong.
+           * tp_account_reconnect_async() won't do anything if the requested
+           * presence is offline anyway. */
+          if (tp_account_get_connection_status (account, NULL) ==
+              TP_CONNECTION_STATUS_DISCONNECTED)
+            reconnect_required = TRUE;
 
-          enabled_checked =
-#ifdef HAVE_MEEGO
-            mx_gtk_light_switch_get_active (
-                MX_GTK_LIGHT_SWITCH (priv->enabled_checkbox));
-#else
-            gtk_toggle_button_get_active (
-                GTK_TOGGLE_BUTTON (priv->enabled_checkbox));
-#endif /* HAVE_MEEGO */
-
-          if (tp_account_is_enabled (account) && enabled_checked)
+          if (reconnect_required && tp_account_is_enabled (account)
+              && priv->enabled)
             {
               /* After having applied changes to a user account, we
-               * automatically reconnect it. This is done so the new
+               * reconnect it if needed. This is done so the new
                * information entered by the user is validated on the server. */
               tp_account_reconnect_async (account, NULL, NULL);
             }
@@ -776,8 +867,7 @@ account_widget_applied_cb (GObject *source_object,
 }
 
 static void
-account_widget_apply_clicked_cb (GtkWidget *button,
-    EmpathyAccountWidget *self)
+account_widget_apply_and_log_in (EmpathyAccountWidget *self)
 {
   EmpathyAccountWidgetPriv *priv = GET_PRIV (self);
   gboolean display_name_overridden;
@@ -814,6 +904,13 @@ account_widget_apply_clicked_cb (GtkWidget *button,
       account_widget_applied_cb, self);
 }
 
+static void
+account_widget_apply_clicked_cb (GtkWidget *button,
+    EmpathyAccountWidget *self)
+{
+    account_widget_apply_and_log_in (self);
+}
+
 static void
 account_widget_setup_generic (EmpathyAccountWidget *self)
 {
@@ -904,12 +1001,13 @@ account_widget_build_irc (EmpathyAccountWidget *self,
 
   if (priv->simple)
     {
-      empathy_account_widget_irc_build_simple (self, filename);
+      priv->irc_network_chooser = empathy_account_widget_irc_build_simple (self,
+          filename);
     }
   else
     {
-      empathy_account_widget_irc_build (self, filename,
-        &priv->table_common_settings);
+      priv->irc_network_chooser = empathy_account_widget_irc_build (self,
+          filename, &priv->table_common_settings);
     }
 }
 
@@ -920,6 +1018,17 @@ account_widget_build_sip (EmpathyAccountWidget *self,
   EmpathyAccountWidgetPriv *priv = GET_PRIV (self);
   empathy_account_widget_sip_build (self, filename,
     &priv->table_common_settings);
+
+  if (priv->simple)
+    {
+      priv->remember_password_widget = GTK_WIDGET (gtk_builder_get_object (
+              self->ui_details->gui, "remember_password_simple"));
+    }
+  else
+    {
+      priv->remember_password_widget = GTK_WIDGET (gtk_builder_get_object (
+              self->ui_details->gui, "remember_password"));
+    }
 }
 
 static void
@@ -940,6 +1049,9 @@ account_widget_build_msn (EmpathyAccountWidget *self,
           NULL);
 
       self->ui_details->default_focus = g_strdup ("entry_id_simple");
+
+      priv->remember_password_widget = GTK_WIDGET (gtk_builder_get_object (
+              self->ui_details->gui, "remember_password_simple"));
     }
   else
     {
@@ -956,6 +1068,9 @@ account_widget_build_msn (EmpathyAccountWidget *self,
           NULL);
 
       self->ui_details->default_focus = g_strdup ("entry_id");
+
+      priv->remember_password_widget = GTK_WIDGET (gtk_builder_get_object (
+              self->ui_details->gui, "remember_password"));
     }
 }
 
@@ -1082,6 +1197,9 @@ account_widget_build_jabber (EmpathyAccountWidget *self,
           NULL);
 
       self->ui_details->default_focus = g_strdup ("entry_id_simple");
+
+      priv->remember_password_widget = GTK_WIDGET (gtk_builder_get_object (
+              self->ui_details->gui, "remember_password_simple"));
     }
   else if (priv->simple && is_gtalk)
     {
@@ -1096,6 +1214,9 @@ account_widget_build_jabber (EmpathyAccountWidget *self,
           NULL);
 
       self->ui_details->default_focus = g_strdup ("entry_id_g_simple");
+
+      priv->remember_password_widget = GTK_WIDGET (gtk_builder_get_object (
+              self->ui_details->gui, "remember_password_g_simple"));
     }
   else if (priv->simple && is_facebook)
     {
@@ -1112,6 +1233,9 @@ account_widget_build_jabber (EmpathyAccountWidget *self,
       setup_facebook_id_widget (self, entry_id);
 
       self->ui_details->default_focus = g_strdup ("entry_id_fb_simple");
+
+      priv->remember_password_widget = GTK_WIDGET (gtk_builder_get_object (
+              self->ui_details->gui, "remember_password_fb_simple"));
     }
   else
     {
@@ -1156,6 +1280,9 @@ account_widget_build_jabber (EmpathyAccountWidget *self,
       self->ui_details->default_focus = g_strdup ("entry_id");
       priv->spinbutton_port = spinbutton_port;
 
+      priv->remember_password_widget = GTK_WIDGET (gtk_builder_get_object (
+              self->ui_details->gui, "remember_password"));
+
       g_signal_connect (checkbutton_ssl, "toggled",
           G_CALLBACK (account_widget_jabber_ssl_toggled_cb),
           self);
@@ -1193,6 +1320,9 @@ account_widget_build_icq (EmpathyAccountWidget *self,
           NULL);
 
       self->ui_details->default_focus = g_strdup ("entry_uin_simple");
+
+      priv->remember_password_widget = GTK_WIDGET (gtk_builder_get_object (
+              self->ui_details->gui, "remember_password_simple"));
     }
   else
     {
@@ -1211,6 +1341,9 @@ account_widget_build_icq (EmpathyAccountWidget *self,
           NULL);
 
       self->ui_details->default_focus = g_strdup ("entry_uin");
+
+      priv->remember_password_widget = GTK_WIDGET (gtk_builder_get_object (
+              self->ui_details->gui, "remember_password"));
     }
 }
 
@@ -1233,6 +1366,9 @@ account_widget_build_aim (EmpathyAccountWidget *self,
           NULL);
 
       self->ui_details->default_focus = g_strdup ("entry_screenname_simple");
+
+      priv->remember_password_widget = GTK_WIDGET (gtk_builder_get_object (
+              self->ui_details->gui, "remember_password_simple"));
     }
   else
     {
@@ -1250,6 +1386,9 @@ account_widget_build_aim (EmpathyAccountWidget *self,
           NULL);
 
       self->ui_details->default_focus = g_strdup ("entry_screenname");
+
+      priv->remember_password_widget = GTK_WIDGET (gtk_builder_get_object (
+              self->ui_details->gui, "remember_password"));
     }
 }
 
@@ -1271,6 +1410,9 @@ account_widget_build_yahoo (EmpathyAccountWidget *self,
           NULL);
 
       self->ui_details->default_focus = g_strdup ("entry_id_simple");
+
+      priv->remember_password_widget = GTK_WIDGET (gtk_builder_get_object (
+              self->ui_details->gui, "remember_password_simple"));
     }
   else
     {
@@ -1282,15 +1424,16 @@ account_widget_build_yahoo (EmpathyAccountWidget *self,
       empathy_account_widget_handle_params (self,
           "entry_id", "account",
           "entry_password", "password",
-          "entry_server", "server",
           "entry_locale", "room-list-locale",
           "entry_charset", "charset",
           "spinbutton_port", "port",
-          "checkbutton_yahoojp", "yahoojp",
           "checkbutton_ignore_invites", "ignore-invites",
           NULL);
 
       self->ui_details->default_focus = g_strdup ("entry_id");
+
+      priv->remember_password_widget = GTK_WIDGET (gtk_builder_get_object (
+              self->ui_details->gui, "remember_password"));
     }
 }
 
@@ -1312,6 +1455,9 @@ account_widget_build_groupwise (EmpathyAccountWidget *self,
           NULL);
 
       self->ui_details->default_focus = g_strdup ("entry_id_simple");
+
+      priv->remember_password_widget = GTK_WIDGET (gtk_builder_get_object (
+              self->ui_details->gui, "remember_password_simple"));
     }
   else
     {
@@ -1328,6 +1474,9 @@ account_widget_build_groupwise (EmpathyAccountWidget *self,
           NULL);
 
       self->ui_details->default_focus = g_strdup ("entry_id");
+
+      priv->remember_password_widget = GTK_WIDGET (gtk_builder_get_object (
+              self->ui_details->gui, "remember_password"));
     }
 }
 
@@ -1364,6 +1513,7 @@ empathy_account_widget_enabled_cb (TpAccount *account,
     }
 }
 
+#if 0
 static void
 #ifdef HAVE_MEEGO
 account_widget_switch_flipped_cb (MxGtkLightSwitch *sw,
@@ -1390,6 +1540,7 @@ account_widget_enabled_toggled_cb (GtkToggleButton *toggle_button,
   tp_account_set_enabled_async (account, state,
       account_widget_account_enabled_cb, user_data);
 }
+#endif
 
 void
 empathy_account_widget_set_other_accounts_exist (EmpathyAccountWidget *self,
@@ -1426,6 +1577,9 @@ do_set_property (GObject *object,
       empathy_account_widget_set_other_accounts_exist (
           EMPATHY_ACCOUNT_WIDGET (object), g_value_get_boolean (value));
       break;
+    case PROP_ENABLED:
+      priv->enabled = g_value_get_boolean (value);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
     }
@@ -1457,6 +1611,9 @@ do_get_property (GObject *object,
     case PROP_OTHER_ACCOUNTS_EXIST:
       g_value_set_boolean (value, priv->other_accounts_exist);
       break;
+    case PROP_ENABLED:
+      g_value_set_boolean (value, priv->enabled);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
     }
@@ -1552,6 +1709,7 @@ static void
 add_enable_checkbox (EmpathyAccountWidget *self,
     TpAccount *account)
 {
+#if 0
   EmpathyAccountWidgetPriv *priv = GET_PRIV (self);
 #ifdef HAVE_MEEGO
   GtkWidget *w;
@@ -1614,6 +1772,7 @@ add_enable_checkbox (EmpathyAccountWidget *self,
   g_signal_connect (G_OBJECT (priv->enabled_checkbox), "toggled",
       G_CALLBACK (account_widget_enabled_toggled_cb), self);
 #endif /* HAVE_MEEGO */
+#endif
 }
 
 #ifndef HAVE_MEEGO
@@ -1658,6 +1817,48 @@ add_register_buttons (EmpathyAccountWidget *self,
 }
 #endif /* HAVE_MEEGO */
 
+static void
+remember_password_toggled_cb (GtkToggleButton *button,
+    EmpathyAccountWidget *self)
+{
+  EmpathyAccountWidgetPriv *priv = GET_PRIV (self);
+
+  if (gtk_toggle_button_get_active (button))
+    {
+      gtk_widget_set_sensitive (priv->param_password_widget, TRUE);
+    }
+  else
+    {
+      gtk_widget_set_sensitive (priv->param_password_widget, FALSE);
+      gtk_entry_set_text (GTK_ENTRY (priv->param_password_widget), "");
+      empathy_account_settings_unset (priv->settings, "password");
+    }
+}
+
+static void
+account_settings_password_retrieved_cb (GObject *object,
+    gpointer user_data)
+{
+  EmpathyAccountWidget *self = user_data;
+  EmpathyAccountWidgetPriv *priv = GET_PRIV (self);
+  const gchar *password = empathy_account_settings_get_string (
+      priv->settings, "password");
+
+  if (password != NULL)
+    {
+      /* We have to do this so that when we call gtk_entry_set_text,
+       * the ::changed callback doesn't think the user made the
+       * change. */
+      priv->automatic_change = TRUE;
+      gtk_entry_set_text (GTK_ENTRY (priv->param_password_widget), password);
+      priv->automatic_change = FALSE;
+    }
+
+  gtk_toggle_button_set_active (
+      GTK_TOGGLE_BUTTON (priv->remember_password_widget),
+      !EMP_STR_EMPTY (password));
+}
+
 static void
 do_constructed (GObject *obj)
 {
@@ -1724,6 +1925,42 @@ do_constructed (GObject *obj)
           NULL);
     }
 
+  /* remember password */
+  if (priv->param_password_widget != NULL
+      && priv->remember_password_widget != NULL
+      && empathy_account_settings_supports_sasl (priv->settings))
+    {
+      if (priv->simple)
+        {
+          gtk_toggle_button_set_active (
+              GTK_TOGGLE_BUTTON (priv->remember_password_widget), TRUE);
+        }
+      else
+        {
+          gtk_toggle_button_set_active (
+              GTK_TOGGLE_BUTTON (priv->remember_password_widget),
+              !EMP_STR_EMPTY (empathy_account_settings_get_string (
+                      priv->settings, "password")));
+
+          /* The password might not have been retrieved from the
+           * keyring yet. We should update the remember password
+           * toggle button and the password entry when/if it is. */
+          g_signal_connect (priv->settings, "password-retrieved",
+              G_CALLBACK (account_settings_password_retrieved_cb), self);
+        }
+
+      g_signal_connect (priv->remember_password_widget, "toggled",
+          G_CALLBACK (remember_password_toggled_cb), self);
+
+      remember_password_toggled_cb (
+          GTK_TOGGLE_BUTTON (priv->remember_password_widget), self);
+    }
+  else if (priv->remember_password_widget != NULL
+      && !empathy_account_settings_supports_sasl (priv->settings))
+    {
+      gtk_widget_set_visible (priv->remember_password_widget, FALSE);
+    }
+
   /* dup and init the account-manager */
   priv->account_manager = tp_account_manager_dup ();
 
@@ -1805,7 +2042,8 @@ do_constructed (GObject *obj)
   display_name = empathy_account_settings_get_display_name (priv->settings);
   default_display_name = empathy_account_widget_get_default_display_name (self);
 
-  if (tp_strdiff (display_name, default_display_name))
+  if (tp_strdiff (display_name, default_display_name) &&
+      !priv->creating_account)
     {
       /* The display name of the account is not the one that we'd assign by
        * default; assume that the user changed it manually */
@@ -1906,6 +2144,13 @@ empathy_account_widget_class_init (EmpathyAccountWidgetClass *klass)
   g_object_class_install_property (oclass, PROP_OTHER_ACCOUNTS_EXIST,
                   param_spec);
 
+  param_spec = g_param_spec_boolean ("enabled",
+      "Enabled",
+      "TRUE if this account widget is enabled",
+      FALSE,
+      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+  g_object_class_install_property (oclass, PROP_ENABLED, param_spec);
+
   signals[HANDLE_APPLY] =
     g_signal_new ("handle-apply", G_TYPE_FROM_CLASS (klass),
         G_SIGNAL_RUN_LAST, 0, NULL, NULL,
@@ -2016,17 +2261,19 @@ empathy_account_widget_get_default_display_name (EmpathyAccountWidget *self)
       /* TODO: this should be done in empathy-account-widget-irc */
       if (!tp_strdiff (protocol, "irc"))
         {
-          const gchar* server;
-          server = empathy_account_settings_get_string (priv->settings,
-              "server");
+          EmpathyIrcNetwork *network;
+
+          network = empathy_irc_network_chooser_get_network (
+              priv->irc_network_chooser);
+          g_assert (network != NULL);
 
           /* To translators: The first parameter is the login id and the
-           * second one is the server. The resulting string will be something
-           * like: "MyUserName on chat.freenode.net".
+           * second one is the network. The resulting string will be something
+           * like: "MyUserName on freenode".
            * You should reverse the order of these arguments if the
            * server should come before the login id in your locale.*/
           default_display_name = g_strdup_printf (_("%1$s on %2$s"),
-              login_id, server);
+              login_id, empathy_irc_network_get_name (network));
         }
       else if (account_widget_is_facebook (self))
         {
