@@ -42,13 +42,13 @@
 #include <libempathy/empathy-gsettings.h>
 #include <libempathy/empathy-utils.h>
 #include <libempathy/empathy-dispatcher.h>
-#include <libempathy/empathy-marshal.h>
 
 #include "empathy-chat.h"
 #include "empathy-spell.h"
 #include "empathy-contact-list-store.h"
 #include "empathy-contact-list-view.h"
 #include "empathy-contact-menu.h"
+#include "empathy-gtk-marshal.h"
 #include "empathy-search-bar.h"
 #include "empathy-theme-manager.h"
 #include "empathy-smiley-manager.h"
@@ -631,37 +631,45 @@ typedef struct {
 	gchar *message;
 } ChatCommandMsgData;
 
-#if 0
 static void
-chat_command_msg_cb (EmpathyDispatchOperation *dispatch,
-			      const GError             *error,
+chat_command_msg_cb (GObject *source,
+			      GAsyncResult *result,
 			      gpointer                  user_data)
 {
 	ChatCommandMsgData *data = user_data;
+	GError *error = NULL;
+	TpChannel *channel;
+
+	channel = tp_account_channel_request_ensure_and_observe_channel_finish (
+					TP_ACCOUNT_CHANNEL_REQUEST (source), result, &error);
+
+	if (channel == NULL) {
+		DEBUG ("Failed to get channel: %s", error->message);
+		g_error_free (error);
 
-	if (error != NULL) {
 		empathy_chat_view_append_event (data->chat->view,
 			_("Failed to open private chat"));
 		goto OUT;
 	}
 
-	if (!EMP_STR_EMPTY (data->message)) {
-		EmpathyTpChat *tpchat;
-		EmpathyMessage *message;
+	if (!EMP_STR_EMPTY (data->message) && TP_IS_TEXT_CHANNEL (channel)) {
+		TpTextChannel *text = (TpTextChannel *) channel;
+		TpMessage *msg;
 
-		tpchat = EMPATHY_TP_CHAT (
-			empathy_dispatch_operation_get_channel_wrapper (dispatch));
+		msg = tp_client_message_new_text (TP_CHANNEL_TEXT_MESSAGE_TYPE_NORMAL,
+			data->message);
 
-		message = empathy_message_new (data->message);
-		empathy_tp_chat_send (tpchat, message);
-		g_object_unref (message);
+		tp_text_channel_send_message_async (text, msg, 0, NULL, NULL);
+
+		g_object_unref (msg);
 	}
 
+	g_object_unref (channel);
+
 OUT:
 	g_free (data->message);
 	g_slice_free (ChatCommandMsgData, data);
 }
-#endif
 
 static void
 chat_command_clear (EmpathyChat *chat,
@@ -723,26 +731,36 @@ chat_command_join (EmpathyChat *chat,
 	g_strfreev (rooms);
 }
 
-#if 0
 static void
 chat_command_msg_internal (EmpathyChat *chat,
 			   const gchar *contact_id,
 			   const gchar *message)
 {
 	EmpathyChatPriv *priv = GET_PRIV (chat);
-	TpConnection *connection;
 	ChatCommandMsgData *data;
+	TpAccountChannelRequest *req;
+	GHashTable *request;
+
+	request = tp_asv_new (
+		TP_PROP_CHANNEL_CHANNEL_TYPE, G_TYPE_STRING, TP_IFACE_CHANNEL_TYPE_TEXT,
+		TP_PROP_CHANNEL_TARGET_HANDLE_TYPE, G_TYPE_UINT, TP_HANDLE_TYPE_CONTACT,
+		TP_PROP_CHANNEL_TARGET_ID, G_TYPE_STRING, contact_id,
+		NULL);
+
+	req = tp_account_channel_request_new (priv->account, request,
+		tp_user_action_time_from_x11 (gtk_get_current_event_time ()));
 
 	/* FIXME: We should probably search in members alias. But this
 	 * is enough for IRC */
 	data = g_slice_new (ChatCommandMsgData);
 	data->chat = chat;
 	data->message = g_strdup (message);
-	connection = empathy_tp_chat_get_connection (priv->tp_chat);
-	empathy_dispatcher_chat_with_contact_id (connection, contact_id,
-						 gtk_get_current_event_time (),
-						 chat_command_msg_cb,
-						 data);
+
+	tp_account_channel_request_ensure_and_observe_channel_async (req,
+		NULL, NULL, chat_command_msg_cb, data);
+
+	g_object_unref (req);
+	g_hash_table_unref (request);
 }
 
 static void
@@ -760,7 +778,6 @@ chat_command_msg (EmpathyChat *chat,
 {
 	chat_command_msg_internal (chat, strv[1], strv[2]);
 }
-#endif
 
 static void
 chat_command_nick (EmpathyChat *chat,
@@ -832,14 +849,11 @@ static ChatCommandItem commands[] = {
 	{"j", 2, 2, chat_command_join,
 	 N_("/j <chat room ID>: join a new chat room")},
 
-#if 0
-/* FIXME: https://bugzilla.gnome.org/show_bug.cgi?id=623682 */
 	{"query", 2, 3, chat_command_query,
 	 N_("/query <contact ID> [<message>]: open a private chat")},
 
 	{"msg", 3, 3, chat_command_msg,
 	 N_("/msg <contact ID> <message>: open a private chat")},
-#endif
 
 	{"nick", 2, 2, chat_command_nick,
 	 N_("/nick <nickname>: change your nickname on the current server")},
@@ -1897,7 +1911,7 @@ chat_input_populate_popup_cb (GtkTextView *view,
 
 
 static gboolean
-chat_log_filter (TplEntry *log,
+chat_log_filter (TplEvent *log,
 		 gpointer user_data)
 {
 	EmpathyChat *chat = user_data;
@@ -1905,11 +1919,11 @@ chat_log_filter (TplEntry *log,
 	EmpathyChatPriv *priv = GET_PRIV (chat);
 	const GList *pending;
 
-	g_return_val_if_fail (TPL_IS_ENTRY (log), FALSE);
+	g_return_val_if_fail (TPL_IS_EVENT (log), FALSE);
 	g_return_val_if_fail (EMPATHY_IS_CHAT (chat), FALSE);
 
 	pending = empathy_tp_chat_get_pending_messages (priv->tp_chat);
-	message = empathy_message_from_tpl_log_entry (log);
+	message = empathy_message_from_tpl_log_event (log);
 
 	for (; pending; pending = g_list_next (pending)) {
 		if (empathy_message_equal (message, pending->data)) {
@@ -1956,7 +1970,7 @@ got_filtered_messages_cb (GObject *manager,
 	EmpathyChatPriv *priv = GET_PRIV (chat);
 	GError *error = NULL;
 
-	if (!tpl_log_manager_get_filtered_messages_finish (TPL_LOG_MANAGER (manager),
+	if (!tpl_log_manager_get_filtered_events_finish (TPL_LOG_MANAGER (manager),
 		result, &messages, &error)) {
 		DEBUG ("%s. Aborting.", error->message);
 		empathy_chat_view_append_event (chat->view,
@@ -1967,9 +1981,9 @@ got_filtered_messages_cb (GObject *manager,
 
 	for (l = messages; l; l = g_list_next (l)) {
 		EmpathyMessage *message;
-		g_assert (TPL_IS_ENTRY (l->data));
+		g_assert (TPL_IS_EVENT (l->data));
 
-		message = empathy_message_from_tpl_log_entry (l->data);
+		message = empathy_message_from_tpl_log_event (l->data);
 		g_object_unref (l->data);
 
 		empathy_chat_view_append_message (chat->view, message);
@@ -1995,7 +2009,7 @@ static void
 chat_add_logs (EmpathyChat *chat)
 {
 	EmpathyChatPriv *priv = GET_PRIV (chat);
-	gboolean         is_chatroom;
+	TplEntity       *target;
 
 	if (!priv->id) {
 		return;
@@ -2005,18 +2019,23 @@ chat_add_logs (EmpathyChat *chat)
 	empathy_chat_view_scroll (chat->view, FALSE);
 
 	/* Add messages from last conversation */
-	is_chatroom = priv->handle_type == TP_HANDLE_TYPE_ROOM;
+	if (priv->handle_type == TP_HANDLE_TYPE_ROOM)
+	  target = tpl_entity_new_from_room_id (priv->id);
+	else
+	  target = tpl_entity_new (priv->id, TPL_ENTITY_CONTACT, NULL, NULL);
 
 	priv->retrieving_backlogs = TRUE;
-	tpl_log_manager_get_filtered_messages_async (priv->log_manager,
-							      priv->account,
-							      priv->id,
-							      is_chatroom,
-							      5,
-							      chat_log_filter,
-							      chat,
-							      got_filtered_messages_cb,
-							      (gpointer) chat);
+	tpl_log_manager_get_filtered_events_async (priv->log_manager,
+						   priv->account,
+						   target,
+						   TPL_EVENT_MASK_TEXT,
+						   5,
+						   chat_log_filter,
+						   chat,
+						   got_filtered_messages_cb,
+						   (gpointer) chat);
+
+	g_object_unref (target);
 }
 
 static gint
@@ -2426,6 +2445,7 @@ chat_create_ui (EmpathyChat *chat)
 	gchar           *filename;
 	GtkTextBuffer   *buffer;
 	gint              paned_pos;
+	EmpathyThemeManager *theme_mgr;
 
 	filename = empathy_file_lookup ("empathy-chat.ui",
 					"libempathy-gtk");
@@ -2451,7 +2471,9 @@ chat_create_ui (EmpathyChat *chat)
 	g_free (filename);
 
 	/* Add message view. */
-	chat->view = empathy_theme_manager_create_view (empathy_theme_manager_get ());
+	theme_mgr = empathy_theme_manager_dup_singleton ();
+	chat->view = empathy_theme_manager_create_view (theme_mgr);
+	g_object_unref (theme_mgr);
 	/* If this is a GtkTextView, it's set as a drag destination for text/plain
 	   and other types, even though it's non-editable and doesn't accept any
 	   drags.  This steals drag motion for anything inside the scrollbars,
@@ -2756,7 +2778,7 @@ empathy_chat_class_init (EmpathyChatClass *klass)
 			      G_SIGNAL_RUN_LAST,
 			      0,
 			      NULL, NULL,
-			      _empathy_marshal_VOID__OBJECT_BOOLEAN,
+			      _empathy_gtk_marshal_VOID__OBJECT_BOOLEAN,
 			      G_TYPE_NONE,
 			      2, EMPATHY_TYPE_MESSAGE, G_TYPE_BOOLEAN);
 
@@ -3171,7 +3193,8 @@ empathy_chat_get_contact_menu (EmpathyChat *chat)
 		menu = empathy_contact_menu_new (priv->remote_contact,
 						 EMPATHY_CONTACT_FEATURE_CALL |
 						 EMPATHY_CONTACT_FEATURE_LOG |
-						 EMPATHY_CONTACT_FEATURE_INFO);
+						 EMPATHY_CONTACT_FEATURE_INFO |
+						 EMPATHY_CONTACT_FEATURE_BLOCK);
 	}
 	else if (priv->contact_list_view) {
 		EmpathyContactListView *view;
