@@ -32,12 +32,12 @@
 #include <pango/pango.h>
 #include <gdk/gdk.h>
 
+#include <libempathy/empathy-gsettings.h>
 #include <libempathy/empathy-time.h>
 #include <libempathy/empathy-utils.h>
 
 #include "empathy-theme-adium.h"
 #include "empathy-smiley-manager.h"
-#include "empathy-conf.h"
 #include "empathy-ui-utils.h"
 #include "empathy-plist.h"
 #include "empathy-string-parser.h"
@@ -61,10 +61,10 @@ typedef struct {
 	EmpathyContact       *last_contact;
 	time_t                last_timestamp;
 	gboolean              last_is_backlog;
-	gboolean              page_loaded;
+	guint                 pages_loading;
 	GList                *message_queue;
-	guint                 notify_enable_webkit_developer_tools_id;
 	GtkWidget            *inspector_window;
+	GSettings            *gsettings_chat;
 } EmpathyThemeAdiumPriv;
 
 struct _EmpathyAdiumData {
@@ -111,14 +111,13 @@ G_DEFINE_TYPE_WITH_CODE (EmpathyThemeAdium, empathy_theme_adium,
 static void
 theme_adium_update_enable_webkit_developer_tools (EmpathyThemeAdium *theme)
 {
+	EmpathyThemeAdiumPriv *priv = GET_PRIV (theme);
 	WebKitWebView  *web_view = WEBKIT_WEB_VIEW (theme);
 	gboolean        enable_webkit_developer_tools;
 
-	if (!empathy_conf_get_bool (empathy_conf_get (),
-				    EMPATHY_PREFS_CHAT_WEBKIT_DEVELOPER_TOOLS,
-				    &enable_webkit_developer_tools)) {
-		return;
-	}
+	enable_webkit_developer_tools = g_settings_get_boolean (
+			priv->gsettings_chat,
+			EMPATHY_PREFS_CHAT_WEBKIT_DEVELOPER_TOOLS);
 
 	g_object_set (G_OBJECT (webkit_web_view_get_settings (web_view)),
 		      "enable-developer-extras",
@@ -127,7 +126,7 @@ theme_adium_update_enable_webkit_developer_tools (EmpathyThemeAdium *theme)
 }
 
 static void
-theme_adium_notify_enable_webkit_developer_tools_cb (EmpathyConf *conf,
+theme_adium_notify_enable_webkit_developer_tools_cb (GSettings   *gsettings,
 						     const gchar *key,
 						     gpointer     user_data)
 {
@@ -257,13 +256,10 @@ theme_adium_parse_body (const gchar *text)
 {
 	EmpathyStringParser *parsers;
 	GString *string;
-	gboolean use_smileys;
+	GSettings *gsettings = g_settings_new (EMPATHY_PREFS_CHAT_SCHEMA);
 
 	/* Check if we have to parse smileys */
-	empathy_conf_get_bool (empathy_conf_get (),
-			       EMPATHY_PREFS_CHAT_SHOW_SMILEYS,
-			       &use_smileys);
-	if (use_smileys)
+	if (g_settings_get_boolean (gsettings, EMPATHY_PREFS_CHAT_SHOW_SMILEYS))
 		parsers = string_parsers_with_smiley;
 	else
 		parsers = string_parsers;
@@ -274,6 +270,8 @@ theme_adium_parse_body (const gchar *text)
 	string = g_string_sized_new (strlen (text));
 	empathy_string_parser_substr (text, -1, parsers, string);
 
+	g_object_unref (gsettings);
+
 	return g_string_free (string, FALSE);
 }
 
@@ -326,7 +324,8 @@ theme_adium_append_html (EmpathyThemeAdium *theme,
 		         const gchar       *contact_id,
 		         const gchar       *service_name,
 		         const gchar       *message_classes,
-		         time_t             timestamp)
+		         time_t             timestamp,
+		         gboolean           is_backlog)
 {
 	GString     *string;
 	const gchar *cur = NULL;
@@ -382,8 +381,13 @@ theme_adium_append_html (EmpathyThemeAdium *theme,
 				cur++;
 			}
 
-			dup_replace = empathy_time_to_string_local (timestamp,
-				format ? format : EMPATHY_TIME_FORMAT_DISPLAY_SHORT);
+			if (is_backlog) {
+				dup_replace = empathy_time_to_string_local (timestamp,
+					format ? format : EMPATHY_TIME_DATE_FORMAT_DISPLAY_SHORT);
+			} else {
+				dup_replace = empathy_time_to_string_local (timestamp,
+					format ? format : EMPATHY_TIME_FORMAT_DISPLAY_SHORT);
+			}
 			replace = dup_replace;
 			g_free (format);
 		} else {
@@ -414,7 +418,7 @@ theme_adium_append_event_escaped (EmpathyChatView *view,
 					 priv->data->status_html,
 					 priv->data->status_len,
 					 escaped, NULL, NULL, NULL, NULL,
-					 "event", empathy_time_get_current ());
+					 "event", empathy_time_get_current (), FALSE);
 	}
 
 	/* There is no last contact */
@@ -447,7 +451,7 @@ theme_adium_append_message (EmpathyChatView *view,
 	gboolean               is_backlog;
 	gboolean               consecutive;
 
-	if (!priv->page_loaded) {
+	if (priv->pages_loading != 0) {
 		priv->message_queue = g_list_prepend (priv->message_queue,
 						      g_object_ref (msg));
 		return;
@@ -463,7 +467,7 @@ theme_adium_append_message (EmpathyChatView *view,
 	timestamp = empathy_message_get_timestamp (msg);
 	body = empathy_message_get_body (msg);
 	body_escaped = theme_adium_parse_body (body);
-	name = empathy_contact_get_name (sender);
+	name = empathy_contact_get_alias (sender);
 	contact_id = empathy_contact_get_id (sender);
 
 	/* If this is a /me, append an event */
@@ -598,7 +602,7 @@ theme_adium_append_message (EmpathyChatView *view,
 		theme_adium_append_html (theme, func, html, len, body_escaped,
 					 avatar_filename, name, contact_id,
 					 service_name, message_classes->str,
-					 timestamp);
+					 timestamp, is_backlog);
 	} else {
 		DEBUG ("Couldn't find HTML file for this message");
 	}
@@ -652,7 +656,7 @@ theme_adium_clear (EmpathyChatView *view)
 	EmpathyThemeAdiumPriv *priv = GET_PRIV (view);
 	gchar *basedir_uri;
 
-	priv->page_loaded = FALSE;
+	priv->pages_loading++;
 	basedir_uri = g_strconcat ("file://", priv->data->basedir, NULL);
 	webkit_web_view_load_html_string (WEBKIT_WEB_VIEW (view),
 					  priv->data->template_html,
@@ -856,7 +860,10 @@ theme_adium_load_finished_cb (WebKitWebView  *view,
 	EmpathyChatView       *chat_view = EMPATHY_CHAT_VIEW (view);
 
 	DEBUG ("Page loaded");
-	priv->page_loaded = TRUE;
+	priv->pages_loading--;
+
+	if (priv->pages_loading != 0)
+		return;
 
 	/* Display queued messages */
 	priv->message_queue = g_list_reverse (priv->message_queue);
@@ -875,9 +882,7 @@ theme_adium_finalize (GObject *object)
 	EmpathyThemeAdiumPriv *priv = GET_PRIV (object);
 
 	empathy_adium_data_unref (priv->data);
-
-	empathy_conf_notify_remove (empathy_conf_get (),
-				    priv->notify_enable_webkit_developer_tools_id);
+	g_object_unref (priv->gsettings_chat);
 
 	G_OBJECT_CLASS (empathy_theme_adium_parent_class)->finalize (object);
 }
@@ -1066,6 +1071,8 @@ theme_adium_constructed (GObject *object)
 			  object);
 
 	/* Load template */
+	priv->pages_loading = 1;
+
 	basedir_uri = g_strconcat ("file://", priv->data->basedir, NULL);
 	webkit_web_view_load_html_string (WEBKIT_WEB_VIEW (object),
 					  priv->data->template_html,
@@ -1154,11 +1161,11 @@ empathy_theme_adium_init (EmpathyThemeAdium *theme)
 			  G_CALLBACK (theme_adium_navigation_policy_decision_requested_cb),
 			  NULL);
 
-	priv->notify_enable_webkit_developer_tools_id =
-		empathy_conf_notify_add (empathy_conf_get (),
-					 EMPATHY_PREFS_CHAT_WEBKIT_DEVELOPER_TOOLS,
-					 theme_adium_notify_enable_webkit_developer_tools_cb,
-					 theme);
+	priv->gsettings_chat = g_settings_new (EMPATHY_PREFS_CHAT_SCHEMA);
+	g_signal_connect (priv->gsettings_chat,
+		"changed::" EMPATHY_PREFS_CHAT_WEBKIT_DEVELOPER_TOOLS,
+		G_CALLBACK (theme_adium_notify_enable_webkit_developer_tools_cb),
+		theme);
 
 	theme_adium_update_enable_webkit_developer_tools (theme);
 }
