@@ -389,6 +389,7 @@ empathy_individual_manager_add_from_contact (EmpathyIndividualManager *self,
     EmpathyContact *contact)
 {
   EmpathyIndividualManagerPriv *priv;
+  EmpathyContactManager *contact_manager;
   FolksBackendStore *backend_store;
   FolksBackend *backend;
   FolksPersonaStore *persona_store;
@@ -443,6 +444,19 @@ empathy_individual_manager_add_from_contact (EmpathyIndividualManager *self,
 
   g_hash_table_destroy (details);
 
+  /* unblock the EmpathyContact */
+  contact_manager = empathy_contact_manager_dup_singleton ();
+
+  if (empathy_contact_manager_get_flags_for_connection (contact_manager,
+        empathy_contact_get_connection (contact)) &
+      EMPATHY_CONTACT_LIST_CAN_BLOCK)
+    {
+      empathy_contact_list_set_blocked (EMPATHY_CONTACT_LIST (contact_manager),
+          contact, FALSE, FALSE);
+    }
+
+  g_object_unref (contact_manager);
+
 finish:
   tp_clear_object (&backend);
   tp_clear_object (&backend_store);
@@ -548,16 +562,23 @@ empathy_individual_manager_set_blocked (EmpathyIndividualManager *self,
     {
       TpfPersona *persona = l->data;
       EmpathyContact *contact;
+      EmpathyIndividualManagerFlags flags;
 
       if (!TPF_IS_PERSONA (persona))
         continue;
 
       contact = empathy_contact_dup_from_tp_contact (
           tpf_persona_get_contact (persona));
-      empathy_contact_set_persona (contact, FOLKS_PERSONA (persona));
-      empathy_contact_list_set_blocked (
-          EMPATHY_CONTACT_LIST (priv->contact_manager),
-          contact, blocked, abusive);
+      flags = empathy_individual_manager_get_flags_for_connection (self,
+          empathy_contact_get_connection (contact));
+
+      if (flags & EMPATHY_INDIVIDUAL_MANAGER_CAN_BLOCK)
+        {
+          empathy_contact_set_persona (contact, FOLKS_PERSONA (persona));
+          empathy_contact_list_set_blocked (
+              EMPATHY_CONTACT_LIST (priv->contact_manager),
+              contact, blocked, abusive);
+        }
 
       g_object_unref (contact);
     }
