@@ -155,13 +155,13 @@ tp_chat_add (EmpathyContactList *list,
 		object_path = tp_proxy_get_object_path (priv->channel);
 
 		props = tp_asv_new (
-		    TP_IFACE_CHANNEL ".ChannelType", G_TYPE_STRING,
+		    TP_PROP_CHANNEL_CHANNEL_TYPE, G_TYPE_STRING,
 		        TP_IFACE_CHANNEL_TYPE_TEXT,
-		    TP_IFACE_CHANNEL ".TargetHandleType", G_TYPE_UINT,
+		    TP_PROP_CHANNEL_TARGET_HANDLE_TYPE, G_TYPE_UINT,
 		        TP_HANDLE_TYPE_NONE,
-		    EMP_IFACE_CHANNEL_INTERFACE_CONFERENCE ".InitialChannels",
+		    TP_PROP_CHANNEL_INTERFACE_CONFERENCE_INITIAL_CHANNELS,
 		        TP_ARRAY_TYPE_OBJECT_PATH_LIST, &channels,
-		    EMP_IFACE_CHANNEL_INTERFACE_CONFERENCE ".InitialInviteeIDs",
+		    TP_PROP_CHANNEL_INTERFACE_CONFERENCE_INITIAL_INVITEE_IDS,
 		        G_TYPE_STRV, invitees,
 		    /* FIXME: InvitationMessage ? */
 		    NULL);
@@ -338,7 +338,8 @@ tp_chat_received_cb (TpChannel   *channel,
 		return;
 	}
 
- 	DEBUG ("Message received: %s", message_body);
+	DEBUG ("Message received from channel %s: %s",
+		tp_proxy_get_object_path (channel), message_body);
 
 	if (message_flags & TP_CHANNEL_TEXT_MESSAGE_FLAG_NON_TEXT_CONTENT &&
 	    !tp_strdiff (message_body, "")) {
@@ -907,7 +908,7 @@ tp_chat_update_remote_contact (EmpathyTpChat *chat)
 	 * have the group interface. If it has the conference interface, then
 	 * it is indeed a MUC. */
 	if (tp_proxy_has_interface_by_id (priv->channel,
-					  EMP_IFACE_QUARK_CHANNEL_INTERFACE_CONFERENCE)) {
+					  TP_IFACE_QUARK_CHANNEL_INTERFACE_CONFERENCE)) {
 		return;
 	}
 
@@ -1089,7 +1090,7 @@ tp_chat_got_renamed_contacts_cb (TpConnection            *connection,
 
 	if (priv->user == old) {
 		/* We change our nick */
-		g_object_unref (priv->user);
+		tp_clear_object (&priv->user);
 		priv->user = g_object_ref (new);
 	}
 
@@ -1320,7 +1321,7 @@ tp_chat_constructor (GType                  type,
 			const char **oprops = g_value_get_boxed (
 				g_value_array_get_nth (array, 1));
 
-			if (tp_strv_contains (oprops, EMP_IFACE_CHANNEL_INTERFACE_CONFERENCE ".InitialChannels")) {
+			if (tp_strv_contains (oprops, TP_PROP_CHANNEL_INTERFACE_CONFERENCE_INITIAL_CHANNELS)) {
 				priv->can_upgrade_to_muc = TRUE;
 				break;
 			}
