@@ -331,6 +331,11 @@ accounts_dialog_update_status_infobar (EmpathyAccountsDialog *dialog,
   g_signal_handlers_unblock_by_func (priv->enabled_switch,
       accounts_dialog_enable_switch_flipped, dialog);
 
+  /* inform the account widget */
+  g_object_set (priv->setting_widget_object,
+      "enabled", account_enabled,
+      NULL);
+
   if (account_enabled)
     {
       switch (status)
@@ -690,11 +695,22 @@ accounts_dialog_setup_ui_to_add_account (EmpathyAccountsDialog *dialog)
           "talkx.l.google.com:80",
           NULL};
 
+      gchar *extra_certificate_identities[] = {
+          "talk.google.com",
+          NULL};
+
       empathy_account_settings_set_icon_name_async (settings, "im-google-talk",
           NULL, NULL);
 
       empathy_account_settings_set_strv (settings, "fallback-servers",
           fallback_servers);
+
+      if (empathy_account_settings_have_tp_param (settings,
+              "extra-certificate-identities"))
+        {
+          empathy_account_settings_set_strv (settings,
+                  "extra-certificate-identities", extra_certificate_identities);
+        }
     }
   else if (is_facebook)
     {
@@ -1604,6 +1620,28 @@ accounts_dialog_get_account_iter (EmpathyAccountsDialog *dialog,
   return FALSE;
 }
 
+static void
+select_and_scroll_to_iter (EmpathyAccountsDialog *dialog,
+    GtkTreeIter *iter)
+{
+  EmpathyAccountsDialogPriv *priv = GET_PRIV (dialog);
+  GtkTreeSelection *selection;
+  GtkTreePath *path;
+  GtkTreeModel *model;
+
+  selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (priv->treeview));
+
+  gtk_tree_selection_select_iter (selection, iter);
+
+  model = gtk_tree_view_get_model (GTK_TREE_VIEW (priv->treeview));
+  path = gtk_tree_model_get_path (model, iter);
+
+  gtk_tree_view_scroll_to_cell (GTK_TREE_VIEW (priv->treeview), path, NULL,
+      TRUE, 0, 0.5);
+
+  gtk_tree_path_free (path);
+}
+
 static void
 accounts_dialog_model_set_selected (EmpathyAccountsDialog *dialog,
     EmpathyAccountSettings *settings)
@@ -1614,7 +1652,7 @@ accounts_dialog_model_set_selected (EmpathyAccountsDialog *dialog,
 
   selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (priv->treeview));
   if (accounts_dialog_get_settings_iter (dialog, settings, &iter))
-    gtk_tree_selection_select_iter (selection, &iter);
+    select_and_scroll_to_iter (dialog, &iter);
 }
 static void
 accounts_dialog_add (EmpathyAccountsDialog *dialog,
@@ -1761,6 +1799,7 @@ accounts_dialog_add_account (EmpathyAccountsDialog *dialog,
   const gchar        *name;
   gboolean            enabled;
   EmpathyAccountsDialogPriv *priv = GET_PRIV (dialog);
+  gboolean selected = FALSE;
 
   model = gtk_tree_view_get_model (GTK_TREE_VIEW (priv->treeview));
   status = tp_account_get_connection_status (account, NULL);
@@ -1770,7 +1809,16 @@ accounts_dialog_add_account (EmpathyAccountsDialog *dialog,
   settings = empathy_account_settings_new_for_account (account);
 
   if (!accounts_dialog_get_account_iter (dialog, account, &iter))
-    gtk_list_store_append (GTK_LIST_STORE (model), &iter);
+    {
+      gtk_list_store_append (GTK_LIST_STORE (model), &iter);
+    }
+  else
+    {
+      GtkTreeSelection *selection;
+
+      selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (priv->treeview));
+      selected = gtk_tree_selection_iter_is_selected (selection, &iter);
+    }
 
   gtk_list_store_set (GTK_LIST_STORE (model), &iter,
       COL_NAME, name,
@@ -1779,6 +1827,14 @@ accounts_dialog_add_account (EmpathyAccountsDialog *dialog,
       COL_ACCOUNT_SETTINGS, settings,
       -1);
 
+  if (selected)
+    {
+      /* We just modified the selected account. Its display name may have been
+       * changed and so it's place in the treeview. Scroll to it so it stays
+       * visible. */
+      select_and_scroll_to_iter (dialog, &iter);
+    }
+
   accounts_dialog_connection_changed_cb (account,
       0,
       status,
@@ -1952,20 +2008,7 @@ accounts_dialog_set_selected_account (EmpathyAccountsDialog *dialog,
 
   selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (priv->treeview));
   if (accounts_dialog_get_account_iter (dialog, account, &iter))
-    {
-      GtkTreePath *path;
-      GtkTreeModel *model;
-
-      gtk_tree_selection_select_iter (selection, &iter);
-
-      model = gtk_tree_view_get_model (GTK_TREE_VIEW (priv->treeview));
-      path = gtk_tree_model_get_path (model, &iter);
-
-      gtk_tree_view_scroll_to_cell (GTK_TREE_VIEW (priv->treeview), path, NULL,
-          TRUE, 0, 0.5);
-
-      gtk_tree_path_free (path);
-    }
+    select_and_scroll_to_iter (dialog, &iter);
 }
 
 static void
