@@ -51,18 +51,23 @@ struct _EmpathyEventManagerClass {
 typedef enum {
     EMPATHY_EVENT_TYPE_CHAT,
     EMPATHY_EVENT_TYPE_VOIP,
+    EMPATHY_EVENT_TYPE_CALL,
     EMPATHY_EVENT_TYPE_TRANSFER,
     EMPATHY_EVENT_TYPE_SUBSCRIPTION,
-    EMPATHY_EVENT_TYPE_PRESENCE
+    EMPATHY_EVENT_TYPE_PRESENCE,
+    EMPATHY_EVENT_TYPE_INVITATION,
+    EMPATHY_EVENT_TYPE_AUTH,
 } EmpathyEventType;
 
 typedef struct {
+    TpAccount *account;
     EmpathyContact *contact;
     EmpathyEventType type;
     gchar *icon_name;
     gchar *header;
     gchar *message;
     gboolean must_ack;
+    GObject *handler_instance;
 } EmpathyEvent;
 
 GType                empathy_event_manager_get_type      (void) G_GNUC_CONST;
@@ -71,6 +76,8 @@ EmpathyEvent *       empathy_event_manager_get_top_event (EmpathyEventManager *m
 GSList *             empathy_event_manager_get_events    (EmpathyEventManager *manager);
 void                 empathy_event_activate              (EmpathyEvent        *event);
 void                 empathy_event_inhibit_updates       (EmpathyEvent        *event);
+void                 empathy_event_approve               (EmpathyEvent        *event);
+void                 empathy_event_decline               (EmpathyEvent        *event);
 
 G_END_DECLS
 
