@@ -51,6 +51,7 @@
 #include <libempathy-gtk/empathy-contact-list-store.h>
 #include <libempathy-gtk/empathy-contact-list-view.h>
 #include <libempathy-gtk/empathy-live-search.h>
+#include <libempathy-gtk/empathy-contact-blocking-dialog.h>
 #include <libempathy-gtk/empathy-contact-search-dialog.h>
 #include <libempathy-gtk/empathy-geometry.h>
 #include <libempathy-gtk/empathy-gtk-enum-types.h>
@@ -65,6 +66,7 @@
 #include <libempathy-gtk/empathy-ui-utils.h>
 
 #include "empathy-accounts-dialog.h"
+#include "empathy-call-observer.h"
 #include "empathy-chat-manager.h"
 #include "empathy-main-window.h"
 #include "empathy-preferences.h"
@@ -107,6 +109,7 @@ struct _EmpathyMainWindowPriv {
 	TpAccountManager        *account_manager;
 	EmpathyChatroomManager  *chatroom_manager;
 	EmpathyEventManager     *event_manager;
+	EmpathyCallObserver     *call_observer;
 	guint                    flash_timeout_id;
 	gboolean                 flash_on;
 	gboolean                 empty;
@@ -615,6 +618,134 @@ main_window_error_close_clicked_cb (GtkButton         *button,
 	main_window_remove_error (window, account);
 }
 
+static void
+main_window_error_upgrade_sw_clicked_cb (GtkButton         *button,
+					 EmpathyMainWindow *window)
+{
+	TpAccount *account;
+	GtkWidget *dialog;
+
+	account = g_object_get_data (G_OBJECT (button), "account");
+	main_window_remove_error (window, account);
+
+	dialog = gtk_message_dialog_new (GTK_WINDOW (window),
+		GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR,
+		GTK_BUTTONS_OK,
+		_("Sorry, %s accounts can’t be used until your %s software is updated."),
+		tp_account_get_protocol (account),
+		tp_account_get_protocol (account));
+
+	g_signal_connect_swapped (dialog, "response",
+		G_CALLBACK (gtk_widget_destroy),
+		dialog);
+
+	gtk_widget_show (dialog);
+}
+
+static void
+main_window_upgrade_software_error (EmpathyMainWindow *window,
+				    TpAccount         *account)
+{
+	EmpathyMainWindowPriv *priv = GET_PRIV (window);
+	GtkWidget *info_bar;
+	GtkWidget *content_area;
+	GtkWidget *label;
+	GtkWidget *image;
+	GtkWidget *upgrade_button;
+	GtkWidget *close_button;
+	GtkWidget *action_area;
+	GtkWidget *action_table;
+	gchar     *str;
+	const gchar     *icon_name;
+	const gchar *error_message;
+	gboolean user_requested;
+
+	error_message =
+		empathy_account_get_error_message (account, &user_requested);
+
+	if (user_requested) {
+		return;
+	}
+
+	str = g_markup_printf_escaped ("<b>%s</b>\n%s",
+					       tp_account_get_display_name (account),
+					       error_message);
+
+	/* If there are other errors, remove them */
+	main_window_remove_error (window, account);
+
+	info_bar = gtk_info_bar_new ();
+	gtk_info_bar_set_message_type (GTK_INFO_BAR (info_bar), GTK_MESSAGE_ERROR);
+
+	gtk_widget_set_no_show_all (info_bar, TRUE);
+	gtk_box_pack_start (GTK_BOX (priv->errors_vbox), info_bar, FALSE, TRUE, 0);
+	gtk_widget_show (info_bar);
+
+	icon_name = tp_account_get_icon_name (account);
+	image = gtk_image_new_from_icon_name (icon_name, GTK_ICON_SIZE_SMALL_TOOLBAR);
+	gtk_widget_show (image);
+
+	label = gtk_label_new (str);
+	gtk_label_set_use_markup (GTK_LABEL (label), TRUE);
+	gtk_label_set_ellipsize (GTK_LABEL (label), PANGO_ELLIPSIZE_END);
+	gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
+	gtk_widget_show (label);
+	g_free (str);
+
+	content_area = gtk_info_bar_get_content_area (GTK_INFO_BAR (info_bar));
+	gtk_box_pack_start (GTK_BOX (content_area), image, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (content_area), label, TRUE, TRUE, 0);
+
+	image = gtk_image_new_from_stock (GTK_STOCK_REFRESH, GTK_ICON_SIZE_BUTTON);
+	upgrade_button = gtk_button_new ();
+	gtk_button_set_image (GTK_BUTTON (upgrade_button), image);
+	gtk_widget_set_tooltip_text (upgrade_button, _("Update software..."));
+	gtk_widget_show (upgrade_button);
+
+	image = gtk_image_new_from_stock (GTK_STOCK_CLOSE, GTK_ICON_SIZE_BUTTON);
+	close_button = gtk_button_new ();
+	gtk_button_set_image (GTK_BUTTON (close_button), image);
+	gtk_widget_set_tooltip_text (close_button, _("Close"));
+	gtk_widget_show (close_button);
+
+	action_table = gtk_table_new (1, 2, FALSE);
+	gtk_table_set_col_spacings (GTK_TABLE (action_table), 2);
+	gtk_widget_show (action_table);
+
+	action_area = gtk_info_bar_get_action_area (GTK_INFO_BAR (info_bar));
+	gtk_box_pack_start (GTK_BOX (action_area), action_table, FALSE, FALSE, 0);
+
+	gtk_table_attach (GTK_TABLE (action_table), upgrade_button, 0, 1, 0, 1,
+										(GtkAttachOptions) (GTK_SHRINK),
+										(GtkAttachOptions) (GTK_SHRINK), 0, 0);
+	gtk_table_attach (GTK_TABLE (action_table), close_button, 1, 2, 0, 1,
+										(GtkAttachOptions) (GTK_SHRINK),
+										(GtkAttachOptions) (GTK_SHRINK), 0, 0);
+
+	g_object_set_data (G_OBJECT (info_bar), "label", label);
+	g_object_set_data_full (G_OBJECT (info_bar),
+				"account", g_object_ref (account),
+				g_object_unref);
+	g_object_set_data_full (G_OBJECT (upgrade_button),
+				"account", g_object_ref (account),
+				g_object_unref);
+	g_object_set_data_full (G_OBJECT (close_button),
+				"account", g_object_ref (account),
+				g_object_unref);
+
+	g_signal_connect (upgrade_button, "clicked",
+			  G_CALLBACK (main_window_error_upgrade_sw_clicked_cb),
+			  window);
+	g_signal_connect (close_button, "clicked",
+			  G_CALLBACK (main_window_error_close_clicked_cb),
+			  window);
+
+	gtk_widget_set_tooltip_text (priv->errors_vbox, error_message);
+	gtk_widget_show (priv->errors_vbox);
+
+	g_hash_table_insert (priv->errors, g_object_ref (account), info_bar);
+}
+
 static void
 main_window_error_display (EmpathyMainWindow *window,
 			   TpAccount         *account)
@@ -634,6 +765,12 @@ main_window_error_display (EmpathyMainWindow *window,
 	const gchar *error_message;
 	gboolean user_requested;
 
+	if (!tp_strdiff (TP_ERROR_STR_SOFTWARE_UPGRADE_REQUIRED,
+			 tp_account_get_detailed_error (account, NULL))) {
+		main_window_upgrade_software_error (window, account);
+		return;
+	}
+
 	error_message =
 		empathy_account_get_error_message (account, &user_requested);
 
@@ -859,6 +996,7 @@ empathy_main_window_finalize (GObject *window)
 	g_signal_handlers_disconnect_by_func (priv->event_manager,
 			  		      main_window_event_removed_cb,
 			  		      window);
+	g_object_unref (priv->call_observer);
 	g_object_unref (priv->event_manager);
 	g_object_unref (priv->ui_manager);
 	g_object_unref (priv->chatroom_manager);
@@ -1435,6 +1573,18 @@ main_window_edit_personal_information_cb (GtkAction         *action,
 	empathy_contact_personal_dialog_show (GTK_WINDOW (window));
 }
 
+static void
+main_window_edit_blocked_contacts_cb (GtkAction         *action,
+		                      EmpathyMainWindow *window)
+{
+	GtkWidget *dialog;
+
+	dialog = empathy_contact_blocking_dialog_new (GTK_WINDOW (window));
+	gtk_widget_show (dialog);
+	g_signal_connect (dialog, "response",
+			G_CALLBACK (gtk_widget_destroy), NULL);
+}
+
 static void
 main_window_edit_preferences_cb (GtkAction         *action,
 				 EmpathyMainWindow *window)
@@ -1586,7 +1736,8 @@ main_window_connection_items_setup (EmpathyMainWindow *window,
 		"chat_new_message",
 		"chat_new_call",
 		"chat_add_contact",
-		"edit_personal_information"
+		"edit_personal_information",
+		"edit_blocked_contacts"
 	};
 
 	for (i = 0, list = NULL; i < G_N_ELEMENTS (actions_connected); i++) {
@@ -1764,6 +1915,7 @@ empathy_main_window_init (EmpathyMainWindow *window)
 			      "edit", "activate", main_window_edit_cb,
 			      "edit_accounts", "activate", main_window_edit_accounts_cb,
 			      "edit_personal_information", "activate", main_window_edit_personal_information_cb,
+			      "edit_blocked_contacts", "activate", main_window_edit_blocked_contacts_cb,
 			      "edit_preferences", "activate", main_window_edit_preferences_cb,
 			      "help_about", "activate", main_window_help_about_cb,
 			      "help_debug", "activate", main_window_help_debug_cb,
@@ -1816,6 +1968,7 @@ empathy_main_window_init (EmpathyMainWindow *window)
 	gtk_widget_show (priv->presence_chooser);
 	item = gtk_tool_item_new ();
 	gtk_widget_show (GTK_WIDGET (item));
+	gtk_widget_set_size_request (priv->presence_chooser, 10, -1);
 	gtk_container_add (GTK_CONTAINER (item), priv->presence_chooser);
 	gtk_tool_item_set_is_important (item, TRUE);
 	gtk_tool_item_set_expand (item, TRUE);
@@ -1892,6 +2045,7 @@ empathy_main_window_init (EmpathyMainWindow *window)
 	empathy_geometry_bind (GTK_WINDOW (window), GEOMETRY_NAME);
 
 	/* Enable event handling */
+	priv->call_observer = empathy_call_observer_dup_singleton ();
 	priv->event_manager = empathy_event_manager_dup_singleton ();
 
 	g_signal_connect (priv->event_manager, "event-added",
